#include "clipboardbackend.h"
#include "utils.h"

#include <QBuffer>
#include <QClipboard>
#include <QCursor>
#include <QGuiApplication>
#include <QMimeData>
#include <QAction>
#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>

// kf5
#include <KF5/KGlobalAccel/KGlobalAccel>
#include <KF5/KWindowSystem/KWindowSystem>
#include <X11/extensions/XTest.h>
#include <X11/Xlib.h>
#include <X11/keysymdef.h>
#include <X11/keysym.h>

#include "titlelesswindow.h"

ClipboardBackend::ClipboardBackend(QObject *parent)
    : QObject{parent}
    , m_imageProvider(new InMemoryImageProvider())
{
    qmlRegisterType<TitleLessWindow>("TitleLessWindow", 1, 0, "DeactiveWindow");
    monitorShortCut();
}

void ClipboardBackend::setDisplayedType(int type)
{
    if (m_displayedType == type) {
        return;
    }

    m_displayedType = type;
    emit displayedTypeChanged();
}

void ClipboardBackend::setOriginalType(int type)
{
    if (m_originalType == type) {
        return;
    }

    m_originalType = type;
    emit originalTypeChanged();
}

void ClipboardBackend::setImageSize(QSize size)
{
    if (m_imageSize == size) {
        return;
    }

    m_imageSize = size;
    emit imageSizeChanged();
}


void ClipboardBackend::addContentMenu(ContentMenu *menu)
{
    m_contentMenus.append(menu);
    emit contentMenusChanged();
}

void ClipboardBackend::removeContentMenu(int index)
{
    m_contentMenus.removeAt(index);
    emit contentMenusChanged();
}

void ClipboardBackend::clearContentMenu()
{
    qDeleteAll(m_contentMenus);
    m_contentMenus.clear();
}

InMemoryImageProvider *ClipboardBackend::getImageProvider()
{
    return m_imageProvider;
}

void ClipboardBackend::addImage(const QImage &image)
{
    if (m_imageProvider) {
        m_imageProvider->addImage(image);
    }
}

void ClipboardBackend::clearImages()
{
    if (m_imageProvider) {
        m_imageProvider->clear();
    }
}

bool ClipboardBackend::processClipboardData()
{
    const QClipboard *clipboard = QGuiApplication::clipboard();
    const QMimeData *mimeData = clipboard->mimeData();
    if (mimeData == nullptr) {
        return false;
    }

    this->clearContentMenu();
    this->clearImages();
    if (mimeData->hasImage()) {
        QImage image = qvariant_cast<QImage>(mimeData->imageData());
        this->addImage(image);
        setImageSize(image.size());
        this->m_plainText = convertImageToPlainText(image);
        qDebug()<<"图转文"<<m_plainText;
        setOriginalType(ContentMenu::Image);
        this->addContentMenu(new ContentMenu(ContentMenu::PlainText, "转为纯文本"));
        this->addContentMenu(new ContentMenu(ContentMenu::Original, "原始粘贴"));
    } else if (mimeData->hasHtml()) {
        this->m_richText = mimeData->html();
        this->m_plainText = clipboard->text();
        setOriginalType(ContentMenu::RichText);
        this->addImage(convertRichTextToImage(this->m_richText, QSize(450, 246)));
        setImageSize(QSize(450, 246));
        this->addContentMenu(new ContentMenu(ContentMenu::Image, "转为图片"));
        this->addContentMenu(new ContentMenu(ContentMenu::PlainText, "转为纯文本"));
        this->addContentMenu(new ContentMenu(ContentMenu::Original, "原始粘贴"));
    } else {
        this->m_plainText = clipboard->text();
        setOriginalType(ContentMenu::PlainText);
        this->addImage(convertPlainTextToImage(this->m_plainText, QSize(450, 246)));
        setImageSize(QSize(450, 246));
        this->addContentMenu(new ContentMenu(ContentMenu::Image, "转为图片"));
        this->addContentMenu(new ContentMenu(ContentMenu::Original, "原始粘贴"));
    }
    return true;
}

void ClipboardBackend::open()
{
    if (processClipboardData()) {
        showContentMenu();
    }
}

void ClipboardBackend::monitorShortCut()
{
    // TODO: 监听快捷键
    auto* action = new QAction(this);
    action->setObjectName("ShowContentMenu");
    action->setText("Show Content Menu");

    QKeySequence keySeq(Qt::META + Qt::Key_V);
    KGlobalAccel::self()->setDefaultShortcut(action, QList<QKeySequence>() << keySeq);
    KGlobalAccel::self()->setShortcut(action, QList<QKeySequence>() << keySeq);
    connect(action, &QAction::triggered, this, [this]() {
        if (processClipboardData()) {
            showContentMenu();
        }
    });
}

void ClipboardBackend::showContentMenu()
{
    emit contentMenuVisibleChanged(true);
}

void ClipboardBackend::pasteClicked() const
{
    Display* disp = XOpenDisplay(NULL);
    if(disp == NULL)
    {
        return;
    }
    //模拟Ctrl+C的组合键操作
    int ctrl_value = XK_Control_L;
    int key_value = XK_V;
    KeyCode keycode = 0, modcode = 0;

    keycode = XKeysymToKeycode (disp, key_value);
    if (keycode == 0) return;
    XTestGrabControl (disp, True);

     //保持Ctrl键
    if (ctrl_value != 0)
    {
        modcode = XKeysymToKeycode(disp, ctrl_value);
        XTestFakeKeyEvent (disp, modcode, True, 0);
    }

    //模拟C键按下
    XTestFakeKeyEvent (disp, keycode, True, 0);
    XTestFakeKeyEvent (disp, keycode, False, 0);

    //释放Ctrl键
    if (ctrl_value != 0)
        XTestFakeKeyEvent (disp, modcode, False, 0);

    XSync (disp, False);
    XTestGrabControl (disp, False);

    XCloseDisplay(disp);
}

int ClipboardBackend::imageNumber() const
{
    if (m_imageProvider) {
        return m_imageProvider->imageNumber();
    }

    return 0;
}

void ClipboardBackend::copyToClipboard(int contentType)
{
    QClipboard *clipboard = QGuiApplication::clipboard();
    if (clipboard == nullptr) {
        return;
    }

    if (contentType == ContentMenu::PlainText) {
        clipboard->setText(m_plainText);
    } else if (contentType == ContentMenu::RichText) {
        clipboard->setText(m_richText);
    } else {
        QList<QImage> images = m_imageProvider->requestAllImage();
        clipboard->setImage(images[0]);
    }
}

void ClipboardBackend::performPasteOperation()
{

}

QPoint ClipboardBackend::getGlobalMousePosition() const
{
    return QCursor::pos();
}

void ClipboardBackend::setRichText(const QString &richText)
{
    if (m_richText == richText) {
        return;
    }
    m_richText = richText;
    emit richTextChanged();
}

void ClipboardBackend::setPlainText(const QString &plainText)
{
    if (m_plainText == plainText) {
        return;
    }
    m_plainText = plainText;

    qDebug()<<"m_plainText"<<m_plainText;
    emit plainTextChanged();
}
